from starlette.responses import *
from royalnet.constellation.api import *
from royalnet.utils import *
from ..tables import *


class ApiDiarioListStar(ApiStar):
    path = "/api/diario/list/v1"

    summary = "Get a list of up to 500 diario entries."

    parameters = {
        "page": "The diario page you want to get. Can be negative to get the entries in reverse order."
    }

    tags = ["diario"]

    async def api(self, data: ApiData) -> dict:
        page_str = data["page"]
        try:
            page = int(page_str)
        except ValueError:
            raise InvalidParameterError("'page' is not a valid int.")
        if page < 0:
            page = -page-1
            entries: typing.List[Diario] = await asyncify(
                data.session
                    .query(self.alchemy.get(Diario))
                    .order_by(self.alchemy.get(Diario).diario_id.desc()).limit(500)
                    .offset(page * 500)
                    .all
            )
        else:
            entries: typing.List[Diario] = await asyncify(
                data.session
                    .query(self.alchemy.get(Diario))
                    .order_by(self.alchemy.get(Diario).diario_id)
                    .limit(500)
                    .offset(page * 500)
                    .all
            )
        response = [entry.json() for entry in entries]
        return response



from typing import *
import royalnet
import royalnet.commands as rc
import royalnet.utils as ru
from ..tables import Treasure, FiorygiTransaction
from .magicktreasure import MagicktreasureCommand


class GivetreasureCommand(MagicktreasureCommand):
    name: str = "givetreasure"

    description: str = "Crea un nuovo Treasure di Fiorygi (usando il tuo credito)"

    syntax: str = "{codice} {valore}"

    async def _permission_check(self, author, code, value, data):
        if author.fiorygi.fiorygi < value:
            raise rc.UserError("Non hai abbastanza fiorygi per creare questo Treasure.")

    async def _create_treasure(self, author, code, value, data):
        TreasureT = self.alchemy.get(Treasure)

        treasure = await ru.asyncify(data.session.query(TreasureT).get, code)
        if treasure is not None:
            raise rc.UserError("Esiste già un Treasure con quel codice.")

        treasure = TreasureT(
            code=code,
            value=value,
            redeemed_by=None
        )

        await FiorygiTransaction.spawn_fiorygi(data, author, -value, "aver creato un tesoro")

        return treasure



import royalnet.utils as ru
from royalnet.backpack.tables import *
from royalnet.constellation.api import *
from ..tables import Cvstats


class ApiCvstatsAvgStar(ApiStar):
    path = "/api/cvstats/avg/v1"

    methods = ["GET"]

    summary = "Get some averages on the cvstats."

    tags = ["cvstats"]

    async def api(self, data: ApiData) -> ru.JSON:
        results = data.session.execute("""
SELECT *
FROM (
         SELECT date_part('hour', c.h)   ph,
                AVG(c.members_connected) members_connected,
                AVG(c.users_connected)   users_connected,
                AVG(c.members_online)    members_online,
                AVG(c.users_online)      users_online,
                AVG(c.members_playing)   members_playing,
                AVG(c.users_playing)     users_playing,
                AVG(c.members_total)     members_total,
                AVG(c.users_total)       users_total
         FROM (
                  SELECT date_trunc('hour', c.timestamp) h,
                         AVG(c.members_connected)        members_connected,
                         AVG(c.users_connected)          users_connected,
                         AVG(c.members_online)           members_online,
                         AVG(c.users_online)             users_online,
                         AVG(c.members_playing)          members_playing,
                         AVG(c.users_playing)            users_playing,
                         AVG(c.members_total)            members_total,
                         AVG(c.users_total)              users_total
                  FROM cvstats c
                  GROUP BY h
              ) c
         GROUP BY ph
) all_time
JOIN
(
         SELECT date_part('hour', c.h)   ph,
                AVG(c.members_connected) members_connected,
                AVG(c.users_connected)   users_connected,
                AVG(c.members_online)    members_online,
                AVG(c.users_online)      users_online,
                AVG(c.members_playing)   members_playing,
                AVG(c.users_playing)     users_playing,
                AVG(c.members_total)     members_total,
                AVG(c.users_total)       users_total
         FROM (
                  SELECT date_trunc('hour', c.timestamp) h,
                         AVG(c.members_connected)        members_connected,
                         AVG(c.users_connected)          users_connected,
                         AVG(c.members_online)           members_online,
                         AVG(c.users_online)             users_online,
                         AVG(c.members_playing)          members_playing,
                         AVG(c.users_playing)            users_playing,
                         AVG(c.members_total)            members_total,
                         AVG(c.users_total)              users_total
                  FROM cvstats c
                  WHERE c.timestamp > current_timestamp - interval '7 day'
                  GROUP BY h
              ) c
         GROUP BY ph
) last_week ON last_week.ph = all_time.ph
JOIN
(
         SELECT date_part('hour', c.h)   ph,
                AVG(c.members_connected) members_connected,
                AVG(c.users_connected)   users_connected,
                AVG(c.members_online)    members_online,
                AVG(c.users_online)      users_online,
                AVG(c.members_playing)   members_playing,
                AVG(c.users_playing)     users_playing,
                AVG(c.members_total)     members_total,
                AVG(c.users_total)       users_total
         FROM (
                  SELECT date_trunc('hour', c.timestamp) h,
                         AVG(c.members_connected)        members_connected,
                         AVG(c.users_connected)          users_connected,
                         AVG(c.members_online)           members_online,
                         AVG(c.users_online)             users_online,
                         AVG(c.members_playing)          members_playing,
                         AVG(c.users_playing)            users_playing,
                         AVG(c.members_total)            members_total,
                         AVG(c.users_total)              users_total
                  FROM cvstats c
                  WHERE c.timestamp > current_timestamp - interval '30 day'
                  GROUP BY h
              ) c
         GROUP BY ph
) last_month ON last_month.ph = all_time.ph
JOIN
(
         SELECT date_part('hour', c.h)   ph,
                AVG(c.members_connected) members_connected,
                AVG(c.users_connected)   users_connected,
                AVG(c.members_online)    members_online,
                AVG(c.users_online)      users_online,
                AVG(c.members_playing)   members_playing,
                AVG(c.users_playing)     users_playing,
                AVG(c.members_total)     members_total,
                AVG(c.users_total)       users_total
         FROM (
                  SELECT date_trunc('hour', c.timestamp) h,
                         AVG(c.members_connected)        members_connected,
                         AVG(c.users_connected)          users_connected,
                         AVG(c.members_online)           members_online,
                         AVG(c.users_online)             users_online,
                         AVG(c.members_playing)          members_playing,
                         AVG(c.users_playing)            users_playing,
                         AVG(c.members_total)            members_total,
                         AVG(c.users_total)              users_total
                  FROM cvstats c
                  WHERE c.timestamp > current_timestamp - interval '1 day'
                  GROUP BY h
              ) c
         GROUP BY ph
) last_day ON last_day.ph = all_time.ph;
        """)

        return [{
            "h": r[0],
            "all_time": {
                "members_connected": float(r[1]),
                "users_connected": float(r[2]),
                "members_online": float(r[3]),
                "users_online": float(r[4]),
                "members_playing": float(r[5]),
                "users_playing": float(r[6]),
                "members_total": float(r[7]),
                "users_total": float(r[8])
            },
            "last_week": {
                "members_connected": float(r[10]),
                "users_connected": float(r[11]),
                "members_online": float(r[12]),
                "users_online": float(r[13]),
                "members_playing": float(r[14]),
                "users_playing": float(r[15]),
                "members_total": float(r[16]),
                "users_total": float(r[17])
            },
            "last_month": {
                "members_connected": float(r[19]),
                "users_connected": float(r[20]),
                "members_online": float(r[21]),
                "users_online": float(r[22]),
                "members_playing": float(r[23]),
                "users_playing": float(r[24]),
                "members_total": float(r[25]),
                "users_total": float(r[26])
            },
            "last_day": {
                "members_connected": float(r[28]),
                "users_connected": float(r[29]),
                "members_online": float(r[30]),
                "users_online": float(r[31]),
                "members_playing": float(r[32]),
                "users_playing": float(r[33]),
                "members_total": float(r[34]),
            },
        } for r in sorted(results.fetchall(), key=lambda s: s[0])]



from royalnet.commands import *


class AhnonlosoioCommand(Command):
    name: str = "ahnonlosoio"

    description: str = "Ah, non lo so io!"

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        await data.reply(r"🤷 Ah, non lo so io! ¯\_(ツ)_/¯")


from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.dialects.postgresql import UUID
from ..types import PollMood


class PollComment:
    __tablename__ = "pollcomments"

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True)

    @declared_attr
    def author_id(self):
        return Column(Integer, ForeignKey("users.uid"), nullable=False)

    @declared_attr
    def author(self):
        return relationship("User", backref=backref("poll_comments_created"))

    @declared_attr
    def poll_id(self):
        return Column(UUID(as_uuid=True), ForeignKey("polls.id"))

    @declared_attr
    def poll(self):
        return relationship("Poll", backref=backref("comments"))

    @declared_attr
    def posted(self):
        return Column(DateTime, nullable=False)

    @declared_attr
    def mood(self):
        return Column(Enum(PollMood), nullable=False, default=PollMood.NEUTRAL)

    @declared_attr
    def comment(self):
        return Column(Text)



from typing import *
import royalnet.commands as rc
import royalnet.backpack.tables as rbt

from ..tables import FiorygiTransaction


class GivefiorygiCommand(rc.Command):
    name: str = "givefiorygi"

    description: str = "Cedi fiorygi a un altro utente."

    syntax: str = "{destinatario} {quantità} {motivo}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        author = await data.get_author(error_if_none=True)

        user_arg = args[0]
        qty_arg = args[1]

        if user_arg is None:
            raise rc.InvalidInputError("Non hai specificato un destinatario!")
        user = await rbt.User.find(self.alchemy, data.session, user_arg)
        if user is None:
            raise rc.InvalidInputError("L'utente specificato non esiste!")
        if user.uid == author.uid:
            raise rc.InvalidInputError("Non puoi inviare fiorygi a te stesso!")

        if qty_arg is None:
            raise rc.InvalidInputError("Non hai specificato una quantità!")
        try:
            qty = int(qty_arg)
        except ValueError:
            raise rc.InvalidInputError("La quantità specificata non è un numero!")
        if qty <= 0:
            raise rc.InvalidInputError("La quantità specificata deve essere almeno 1!")

        if author.fiorygi.fiorygi < qty:
            raise rc.InvalidInputError("Non hai abbastanza fiorygi per effettuare la transazione!")

        await FiorygiTransaction.spawn_fiorygi(data, author, -qty, f"aver ceduto fiorygi a {user}")
        await FiorygiTransaction.spawn_fiorygi(data, user, qty, f"aver ricevuto fiorygi da {author}")



from typing import *
from royalnet.commands import *
from royalnet.serf.discord import *
from royalnet.serf.discord.errors import *
from ..utils import RoyalQueue
import discord


class DiscordSummonEvent(Event):
    name = "discord_summon"

    async def run(self, *,
                  channel_name: str = "",
                  channel_id: Optional[int] = None,
                  guild_id: Optional[int] = None,
                  user_id: Optional[int] = None,
                  **kwargs):
        if not isinstance(self.serf, DiscordSerf):
            raise UnsupportedError()
        # Find the guild
        if guild_id is not None:
            guild: Optional[discord.Guild] = self.serf.client.get_guild(guild_id)
        else:
            guild = None
        # Find the member
        if user_id is not None and guild is not None:
            member: Optional[Union[discord.User, discord.Member]] = guild.get_member(user_id=user_id)
        else:
            member = None
        # From channel id
        if channel_id is not None:
            client: discord.Client = self.serf.client
            channel = client.get_channel(channel_id)
        # Find channel
        elif channel_name != "":
            # Find accessible_to
            accessible_to = [self.serf.client.user]
            if member is not None:
                accessible_to.append(member)
            # Find the channel
            channel: Optional["discord.VoiceChannel"] = self.serf.find_channel(channel_type=discord.VoiceChannel,
                                                                               name=channel_name,
                                                                               guild=guild,
                                                                               accessible_to=accessible_to,
                                                                               required_permissions=["connect", "speak"]
                                                                               )
        elif not (guild_id is None and user_id is None):
            if member.voice is not None:
                channel = member.voice.channel
            else:
                channel = None
        else:
            raise InvalidInputError("Non so a cosa devo connettermi! Specifica il nome di un canale, o entra tu stesso"
                                    " in un canale vocale prima di invocare [c]summon[/c]!")
        if channel is None:
            raise InvalidInputError("Non ho trovato nessun canale in cui connettermi.")
        # Create a new VoicePlayer
        vp = VoicePlayer(loop=self.loop)
        vp.playing = await RoyalQueue.create()
        # Connect to the channel
        try:
            await vp.connect(channel)
        except OpusNotLoadedError:
            raise ConfigurationError("libopus non è disponibile sul sistema in cui sta venendo eseguito questo bot,"
                                     " pertanto non è possibile con")
        except DiscordTimeoutError:
            raise ExternalError("Timeout durante la connessione al canale."
                                " Forse il bot non ha i permessi per entrarci?")
        except GuildAlreadyConnectedError:
            raise UserError("Il bot è già connesso in un canale vocale nel Server!\n"
                            "Spostalo manualmente, o disconnettilo e riinvoca [c]summon[/c]!")
        # Add the created VoicePlayer to the list
        self.serf.voice_players.append(vp)
        # Reply to the request
        return {
            "channel": {
                "id": channel.id,
                "name": channel.name,
                "guild": {
                    "name": channel.guild.name,
                },
            }
        }



from sqlalchemy import *
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declared_attr
import steam


class Steam:
    __tablename__ = "steam"

    @declared_attr
    def user_id(self):
        return Column(Integer, ForeignKey("users.uid"))

    @declared_attr
    def user(self):
        return relationship("User", backref=backref("steam"))

    @declared_attr
    def _steamid(self):
        return Column(BigInteger, primary_key=True)

    @property
    def steamid(self):
        return steam.SteamID(self._steamid)

    @declared_attr
    def persona_name(self):
        return Column(String)

    @declared_attr
    def profile_url(self):
        return Column(String)

    @declared_attr
    def avatar(self):
        return Column(String)

    @declared_attr
    def primary_clan_id(self):
        return Column(BigInteger)

    @declared_attr
    def account_creation_date(self):
        return Column(DateTime)

    def __repr__(self):
        return f"<Steam account {self._steamid} of {self.user}>"

    def __str__(self):
        return f"[c]steam:{self._steamid}[/c]"



from typing import *
import royalnet.commands as rc
import discord
import royalnet.backpack.tables as rbt


class PlaymodeCommand(rc.Command):
    name: str = "playmode"

    aliases = ["pm"]

    description: str = "Seleziona la modalità di riproduzione musicale."

    syntax: str = "{queue|pool}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        if self.interface.name == "discord":
            message: discord.Message = data.message
            guild: discord.Guild = message.guild
            if guild is None:
                guild_id = None
            else:
                guild_id: Optional[int] = guild.id
        else:
            guild_id = None

        user: rbt.User = await data.get_author()
        user_str = None

        if user is not None:
            try:
                user_discord: rbt.Discord = user.discord[0]
            except (AttributeError, IndexError):
                user_str = str(user)
            else:
                user_str = str(f"<@{user_discord.discord_id}>")

        response = await self.interface.call_herald_event("discord", "discord_playmode",
                                                          playable_string=args[0],
                                                          guild_id=guild_id,
                                                          user=user_str)

        if response["name"] == "RoyalQueue":
            await data.reply(f"✅ Modalità di riproduzione impostata a [b]Queue[/b]:\n"
                             f"- Riproduci le canzoni nell'ordine scelto\n"
                             f"- Rimuovi le canzoni dopo averle riprodotte")

        elif response["name"] == "RoyalPool":
            await data.reply(f"✅ Modalità di riproduzione impostata a [b]Pool[/b]:\n"
                             f"- Aggiungi canzoni al pool con [c]!p[/c], [c]!yt[/c] e [c]!fw[/c]\n"
                             f"- Riproduci all'infinito canzoni casuali dal pool\n"
                             f"- Non è possibile rimuovere canzoni dal pool, [c]!skip[/c]parle manderà solo avanti il pool\n"
                             f"- Interrompi la riproduzione del pool cambiando modalità di riproduzione")

        else:
            await data.reply(f"✅ Modalità di riproduzione impostata a [c]{response['name']}[/c]!")


class MMInterfaceData:
    def __init__(self):
        pass


class MMInterfaceDataTelegram(MMInterfaceData):
    def __init__(self, chat_id: int, message_id: int):
        super().__init__()
        self.chat_id = chat_id
        self.message_id = message_id
